@@ -5,13 +5,13 @@
 ##-------------------------------------------------------------------------
 
 mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.parents = 1, mcmc.scheme = c(100, 1000,
-    1000), seed = 42, verbose = FALSE, start.dag = NULL, prior.dag = NULL, prior.lambda = NULL, prob.rev = 0.05, prob.mbr = 0.05,
+    1000), seed = 42, verbose = FALSE, start.dag = NULL, prior.dag = NULL, prior.lambda = NULL, prob.rev = 0.05, prob.mbr = 0.05, heating = 1,
     prior.choice = 2) {
 
     #################################################### Tests
 
     .tests.mcmcabn(score.cache, data.dists, max.parents, mcmc.scheme, seed, verbose, start.dag, prior.dag, prior.lambda,
-        prob.rev, prob.mbr, prior.choice)
+        prob.rev, prob.mbr, prior.choice,heating)
 
     ## end of tests
 
@@ -32,16 +32,29 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
 
     prob.mc3 <- 1 - (prob.rev + prob.mbr)
 
+    heating.orig <- NULL
+
     ## output
     out.dags <- array(data = NA, dim = c(n.var, n.var, mcmc.scheme[1] + 1))
     out.scores <- rep(NA, length = mcmc.scheme[1] + 1)
     out.alpha <- rep(NA, length = mcmc.scheme[1] + 1)
     out.method <- rep(NA, length = mcmc.scheme[1] + 1)
     out.rejection <- rep(NA, length = mcmc.scheme[1] + 1)
+    out.heating <- rep(NA, length = mcmc.scheme[1] + 1)
 
     ## seeding
     set.seed(seed = seed)
 
+    ## structural restriction
+
+    retain <- score.cache$dag.retained
+    ban <- score.cache$dag.banned
+
+    if(is.null(retain)){retain <- matrix(data = 0, nrow = n.var, ncol = n.var)}
+
+    if(is.null(ban)){ban <- matrix(data = 0, nrow = n.var, ncol = n.var)}
+
+
     ## Initializing matrix
     dag.tmp <- matrix(data = 0, nrow = n.var, ncol = n.var)
 
@@ -58,6 +71,14 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
         }
         colnames(dag.tmp) <- rownames(dag.tmp) <- name.dag <- sample(1:n.var)
         dag.tmp <- dag.tmp[order(name.dag), order(name.dag)]
+        #check constraints
+        dag.tmp <- dag.tmp + retain
+        dag.tmp <- dag.tmp*(1-ban)
+        ## run a series of checks on the DAG passed
+        dag.tmp <- abs(dag.tmp)
+        ## consistency checks
+        diag(dag.tmp) <- 0
+        dag.tmp[dag.tmp > 0] <- 1
     }
 
     if (is.character(start.dag) && start.dag == "hc"){
@@ -86,10 +107,12 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
         colnames(dag.tmp) <- rownames(dag.tmp) <- 1:n.var
     }
 
+
     ## scores
     if (score %in% c("bic", "aic", "mdl")) {
         tmp <- score.cache$node.defn[, as.numeric(colnames(dag.tmp))]
         colnames(tmp) <- as.numeric(colnames(dag.tmp))
+
         sc <- cbind(tmp, -score.cache[[score]])
 
     }
@@ -97,16 +120,19 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
 
         tmp <- score.cache$node.defn[, as.numeric(colnames(dag.tmp))]
         colnames(tmp) <- as.numeric(colnames(dag.tmp))
+
         sc <- cbind(tmp, score.cache[[score]])
     }
 
-    ## scoring init
-    score.init <- 0
-    for (a in 1:n.var) {
-        sc.tmp <- sc[score.cache$children == as.numeric(colnames(dag.tmp)[a]), ]
-        score.init <- sum(min(sc.tmp[which(apply(sc.tmp, 1, function(x) identical(unname(x[1:n.var]), unname(dag.tmp[a,
-            ])))), n.var + 1]), score.init)
 
+
+    ## scoring init
+    score.init <- score.dag(dag.tmp,score.cache,sc)
+
+    ## heating scheme
+    if(is.numeric(heating) & heating>1){
+      heating.orig <- heating
+      heating <- 1/heating.orig
     }
 
 
@@ -121,46 +147,56 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
 
     out.rejection[1] <- 0
 
+    out.heating[1] <- heating
+
     ## start mcmc search
     if (verbose)
         cat("Start MCMC burn in \n")
 
     j <- 1
 
+
     while (j <= mcmc.scheme[3]) {
 
         ## method choice:
 
         method.choice <- sample(x = c("MC3", "REV", "MBR"), size = 1, prob = c(prob.mc3, prob.rev, prob.mbr))
 
+        if(heating<1 & !is.null(heating.orig)){heating <-1/(heating.orig-j+1)}
+
+
         switch(method.choice, MC3 = {
-            out <- mc3(n.var, (dag.tmp), max.parents, sc, score.cache, score, prior.choice, prior.lambda, prior.dag,
-                verbose)
+            out <- mc3(n.var, (dag.tmp), retain, ban, max.parents, sc, score.cache, score, prior.choice, prior.lambda, prior.dag,
+                verbose, heating)
             dag.tmp <- out$dag.tmp
             score <- out$score
         }, REV = {
             if (verbose) {
                 print("REV move")
             }
-            out <- REV(n.var, (dag.tmp), max.parents, sc, score.cache, score, verbose)
+            out <- REV(n.var, (dag.tmp),retain, ban, max.parents, sc, score.cache, score, verbose, heating)
             dag.tmp <- out$dag.tmp
             score <- out$score
         }, MBR = {
             if (verbose) {
                 print("MBR move")
             }
-            out <- MBR(n.var, dag.tmp, max.parents, sc, score.cache, score, verbose)
+            out <- MBR(n.var, dag.tmp,retain, ban, max.parents, sc, score.cache, score, verbose, heating)
             dag.tmp <- out$dag.tmp
             score <- out$score
         })
 
         j <- j + 1
-    }  #EOF
+    }  #EOF Burn in
 
     out.scores[1] <- score
     out.dags[, , 1] <- dag.tmp
 
-    ### EOF: Burnin phase!
+    ### EOF: Burn-in phase!
+
+    if(!is.null(heating.orig) & heating<1){
+      heating.orig <- round(1/heating)
+    }
 
     if (verbose)
         cat("Start MCMC search \n")
@@ -170,13 +206,16 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
 
         ## start blind search
         j <- 0
+
+        if(heating<1 & !is.null(heating.orig)){heating <- 1/(heating.orig-mcmc.search+1)}
+
         while (j <= mcmc.scheme[2]) {
 
             method.choice <- sample(x = c("MC3", "REV", "MBR"), size = 1, prob = c(prob.mc3, prob.rev, prob.mbr))
 
             switch(method.choice, MC3 = {
-                out <- mc3(n.var, (dag.tmp), max.parents, sc, score.cache, score, prior.choice, prior.lambda, prior.dag,
-                  verbose)
+                out <- mc3(n.var, (dag.tmp), retain, ban, max.parents, sc, score.cache, score, prior.choice, prior.lambda, prior.dag,
+                  verbose, heating)
                 dag.tmp <- out$dag.tmp
                 score <- out$score
                 alpha <- out$alpha
@@ -185,7 +224,7 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
                 if (verbose) {
                   print("REV move")
                 }
-                out <- REV(n.var, (dag.tmp), max.parents, sc, score.cache, score, verbose)
+                out <- REV(n.var, (dag.tmp),retain, ban, max.parents, sc, score.cache, score, verbose, heating)
                 dag.tmp <- out$dag.tmp
                 score <- out$score
                 alpha <- out$alpha
@@ -194,7 +233,7 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
                 if (verbose) {
                   print("MBR move")
                 }
-                out <- MBR(n.var, (dag.tmp), max.parents, sc, score.cache, score, verbose)
+                out <- MBR(n.var, (dag.tmp), retain, ban, max.parents, sc, score.cache, score, verbose, heating)
                 dag.tmp <- out$dag.tmp
                 score <- out$score
                 alpha <- out$alpha
@@ -214,9 +253,11 @@ mcmcabn <- function(score.cache = NULL, score = "mlik", data.dists = NULL, max.p
 
         out.rejection[mcmc.search + 1] <- rejection
 
+        out.heating[mcmc.search + 1] <- heating
+
     }  #EOF mcmc search
     out <- list(dags = out.dags, scores = out.scores, alpha = out.alpha, method = out.method, rejection = out.rejection,
-        iterations = mcmc.scheme[1] * (mcmc.scheme[2] + 1), thinning = mcmc.scheme[2], burnin = mcmc.scheme[3], data.dist = data.dists)
+        iterations = mcmc.scheme[1] * (mcmc.scheme[2] + 1), thinning = mcmc.scheme[2], burnin = mcmc.scheme[3], data.dist = data.dists, heating = out.heating)
 
     class(out) <- "mcmcabn"
 
