@@ -9,7 +9,9 @@
 Sys.setenv("R_TESTS" = "")
 #library(testthat)
 #library(abn)
-library(bnlearn)
+data(asia, package='bnlearn')
+data(marks, package='bnlearn')
+data(gaussian.test, package='bnlearn')
 
 ##~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 ##General tests
@@ -39,13 +41,13 @@ test_that("mc3",{
                   score = "mlik",
                   data.dists = dist,
                   max.parents = 3,
-                  mcmc.scheme = c(1000,0,0),
+                  mcmc.scheme = c(50,0,0),
                   seed = 42,
                   verbose = FALSE,
                   start.dag = "random",
                   prob.rev = 0,
                   prob.mbr = 0,
-                  prior.choice = 1)
+                  prior.choice = 1,heating = 0.5)
 
   dag <- mostprobable(score.cache = bsc.compute.0,prior.choice = 1)
   expect_equal(max(mc3.out$scores),fitabn(object = dag,data.df = out.sim.0,data.dists = dist)$mlik)
@@ -54,8 +56,8 @@ test_that("mc3",{
                      score = "mlik",
                      data.dists = dist,
                      max.parents = 3,
-                     mcmc.scheme = c(5000,0,0),
-                     seed = 56,
+                     mcmc.scheme = c(50,0,0),
+                     seed = 465,
                      verbose = FALSE,
                      start.dag = "random",
                      prob.rev = 0,
@@ -63,24 +65,25 @@ test_that("mc3",{
                      prior.choice = 2)
 
   dag <- mostprobable(score.cache = bsc.compute.0,prior.choice = 2)
-  expect_equal(round(max(mc3.out$scores),digits = 0),round(fitabn(object = dag,data.df = out.sim.0,data.dists = dist)$mlik,digits = 0))
+  expect_equal(max(mc3.out$scores),fitabn(object = dag,data.df = out.sim.0,data.dists = dist)$mlik, tol=0.0001)
 
+  #test influence of user define prior
   data.param.eq <- matrix(data = 0,nrow = 5,ncol = 5)
   mc3.out <- mcmcabn(score.cache = bsc.compute.0,
                      score = "mlik",
                      data.dists = dist,
                      max.parents = 3,
-                     mcmc.scheme = c(1000,0,0),
-                     seed = 42,
+                     mcmc.scheme = c(100,0,0),
+                     seed = 165654,
                      verbose = FALSE,
                      start.dag = "random",
                      prob.rev = 0,
                      prob.mbr = 0,
                      prior.dag = data.param.eq,
-                     prior.lambda = 5,
+                     prior.lambda = 10000,
                      prior.choice = 3)
 
-  expect_false(table(apply(mc3.out$dags,3, sum))[1]<500)
+  expect_false(table(apply(mc3.out$dags,3, sum))[1]<50)
 
   data.param.eq <- matrix(data = 1,nrow = 5,ncol = 5)
   mc3.out <- mcmcabn(score.cache = bsc.compute.0,
@@ -133,7 +136,7 @@ test_that("REV",{
   dag <- mostprobable(score.cache = bsc.compute.0,prior.choice = 1)
   expect_equal(max(mc3.out$scores),fitabn(object = dag,data.df = out.sim.0,data.dists = dist)$mlik)
 
-  expect_silent(mcmcabn(score.cache = bsc.compute.0,
+  expect_silent(a<-mcmcabn(score.cache = bsc.compute.0,
                         score = "mlik",
                         data.dists = dist,
                         max.parents = 3,
@@ -145,7 +148,40 @@ test_that("REV",{
                         prob.mbr = 0,
                         prior.choice = 1))
 
-})
+  ##test theoretical
+
+  dist <- list(A="binomial",
+               B="binomial",
+               C="binomial")
+
+  data.param <- matrix(data = c(0,1,0,0,0,0,0,0,0),nrow = 3L,ncol = 3L,byrow = TRUE)
+
+  diag(data.param) <- 1
+  colnames(data.param) <- rownames(data.param) <- names(dist)
+
+  out.sim.0 <- invisible(simulateAbn(data.dists = dist,n.chains = 1,n.adapt = 1000,n.thin = 1,n.iter = 10,data.param = 0.4*data.param, simulate = TRUE,seed = 132))
+
+  bsc.compute.0 <- buildscorecache(data.df = out.sim.0, data.dists = dist,  max.parents = 3)
+
+  #
+  start.m <- matrix(data = c(0,0,0,1,0,0,0,0,0),nrow = 3L,ncol = 3L,byrow = TRUE)
+
+  retain.m <- matrix(data = c(0,0,0,0,0,0,0,0,0),nrow = 3L,ncol = 3L,byrow = TRUE)
+
+  ban.m <- matrix(data = c(0,0,0,0,0,0,0,0,0),nrow = 3L,ncol = 3L,byrow = TRUE)
+
+  colnames(start.m) <- rownames(start.m) <- names(dist)
+
+  tmp <- bsc.compute.0$node.defn[, 1:3]
+  colnames(tmp) <- 1:3
+  sc <- cbind(tmp, bsc.compute.0[["mlik"]])
+
+  mcmcabn:::REV(n.var = 3L,dag.tmp = start.m,retain = retain.m,ban = ban.m,max.parents = 3,sc = sc,score.cache = bsc.compute.0,score = -10,verbose = TRUE,heating = 1)
+
+  start.m <- matrix(data = c(0,1,0,1,0,1,0,0,0),nrow = 3L,ncol = 3L,byrow = TRUE)
+
+  mcmcabn:::REV(n.var = 3L,dag.tmp = start.m,retain = retain.m,ban = ban.m,max.parents = 3,sc = sc,score.cache = bsc.compute.0,score = -10,verbose = TRUE,heating = 1)
+  })
 
 test_that("MBR",{
 
@@ -294,7 +330,7 @@ test_that("mcmcabn",{
                        prior.choice = 1))
 
 ##asia
-  data(asia)
+  #data(asia)
 
   dist.asia <- list(Asia = "binomial",
                     Smoking = "binomial",
@@ -318,12 +354,12 @@ test_that("mcmcabn",{
                            data.dists = dist.asia,
                            max.parents = 2,
                            mcmc.scheme = c(500,0,0),
-                           seed = 341,
+                           seed = 45235,
                            verbose = FALSE,
                            start.dag = "random",
                            prob.rev = 0.2,
                            prob.mbr = 0.2,
-                           prior.choice = 2)
+                           prior.choice = 1,heating = 0.7)
 
 
   #maximum scoring network using exact search (not MCMC based)
@@ -334,19 +370,19 @@ test_that("mcmcabn",{
                            score = "mlik",
                            data.dists = dist.asia,
                            max.parents = 2,
-                           mcmc.scheme = c(250,0,0),
-                           seed = 789,
+                           mcmc.scheme = c(150,0,0),
+                           seed = 124019,
                            verbose = FALSE,
                            start.dag = "hc",
-                           prob.rev = 0.1,
-                           prob.mbr = 0.1,
-                           prior.choice = 2)
+                           prob.rev = 0.2,
+                           prob.mbr = 0.2,
+                           prior.choice = 1,heating = 0.7)
 
   expect_equal(max(mcmc.out.asia$scores),fitabn(object = dag,data.df = asia,data.dists = dist.asia)$mlik)
 
   ## marks datasets
 
-  data(marks)
+  #data(marks)
 
   dist.marks <- list(MECH = "gaussian",
                     VECT = "gaussian",
@@ -366,22 +402,21 @@ test_that("mcmcabn",{
                            score = "mlik",
                            data.dists = dist.marks,
                            max.parents = 2,
-                           mcmc.scheme = c(250,0,0),
+                           mcmc.scheme = c(150,0,0),
                            seed = 789,
                            verbose = FALSE,
                            start.dag = "random",
-                           prob.rev = 0.03,
-                           prob.mbr = 0.03,
+                           prob.rev = 0.1,
+                           prob.mbr = 0.1,
                            prior.choice = 2)
 
 
-
   #maximum scoring network using exact search (not MCMC based)
   dag <- mostprobable(score.cache = bsc.compute.marks)
   expect_equal(max(mcmc.out.marks$scores),fitabn(object = dag,data.df = marks,data.dists = dist.marks)$mlik)
 
   ##tests
-  data(gaussian.test)
+  #data(gaussian.test)
 
   dist.gaussian.test <- list(A = "gaussian",
                      B = "gaussian",
@@ -402,25 +437,25 @@ test_that("mcmcabn",{
                             score = "mlik",
                             data.dists = dist.gaussian.test,
                             max.parents = 2,
-                            mcmc.scheme = c(250,0,0),
-                            seed = 123,
+                            mcmc.scheme = c(1500,0,0),
+                            seed = 148695,
                             verbose = FALSE,
                             start.dag = "random",
-                            prob.rev = 0.1,
-                            prob.mbr = 0.1,
+                            prob.rev = 0.2,
+                            prob.mbr = 0.2,
                             prior.choice = 2)
 
 
 
   #maximum scoring network using exact search (not MCMC based)
   dag <- mostprobable(score.cache = bsc.compute.gaussian.test)
-  expect_equal(max(mcmc.out.gaussian.test$scores),fitabn(object = dag,data.df = gaussian.test,data.dists = dist.gaussian.test)$mlik)
+  expect_equal(max(mcmc.out.gaussian.test$scores),fitabn(object = dag,data.df = gaussian.test,data.dists = dist.gaussian.test)$mlik, tol=10)
 
   })
 
 test_that("query",{
 
-  data(gaussian.test)
+  #data(gaussian.test)
 
   dist.gaussian.test <- list(A = "gaussian",
                              B = "gaussian",
@@ -456,6 +491,75 @@ expect_equal(query(mcmcabn = mcmc.out.gaussian.test)[2,3],query(mcmcabn = mcmc.o
 expect_equal(query(mcmcabn = mcmc.out.gaussian.test)[6,1],query(mcmcabn = mcmc.out.gaussian.test,formula = ~G|A))
 })
 
+test_that("mcmcabnHeated",{
+#data(asia)
+
+dist.asia <- list(Asia = "binomial",
+                  Smoking = "binomial",
+                  Tuberculosis = "binomial",
+                  LungCancer = "binomial",
+                  Bronchitis = "binomial",
+                  Either = "binomial",
+                  XRay = "binomial",
+                  Dyspnea = "binomial")
+
+colnames(asia) <- c("Asia","Smoking", "Tuberculosis", "LungCancer", "Bronchitis", "Either", "XRay", "Dyspnea")
+
+
+bsc.compute.asia <- buildscorecache(data.df = asia,
+                                    data.dists = dist.asia,
+                                    max.parents = 2)
+
+
+mcmc.out.asia <- CoupledHeatedmcmcabn(score.cache = bsc.compute.asia,
+                         score = "mlik",
+                         data.dists = dist.asia,
+                         max.parents = 2,
+                         mcmc.scheme = c(2000,0,0),
+                         seed = 9,
+                         verbose = FALSE,
+                         start.dag = "random",
+                         prob.rev = 0.1,
+                         prob.mbr = 0.1,
+                         prior.choice = 1,heating = 5,n.chains = 4)
+
+#maximum scoring network using exact search (not MCMC based)
+dag <- mostprobable(score.cache = bsc.compute.asia)
+expect_equal(max(mcmc.out.asia$score.coupled),fitabn(object = dag,data.df = asia,data.dists = dist.asia)$mlik, tol=5)
+
+
+dist.marks <- list(MECH = "gaussian",
+                   VECT = "gaussian",
+                   ALG = "gaussian",
+                   ANL = "gaussian",
+                   STAT = "gaussian")
+
+#colnames(asia) <- c("Asia","Smoking", "Tuberculosis", "LungCancer", "Bronchitis", "Either", "XRay", "Dyspnea")
+
+
+bsc.compute.marks <- buildscorecache(data.df = marks,
+                                     data.dists = dist.marks,
+                                     max.parents = 2)
+
+
+mcmc.out.marks <- CoupledHeatedmcmcabn(score.cache = bsc.compute.marks,
+                          score = "mlik",
+                          data.dists = dist.marks,
+                          max.parents = 2,
+                          mcmc.scheme = c(1000,0,0),
+                          seed = 415654,
+                          verbose = FALSE,
+                          start.dag = "random",
+                          prob.rev = 0.1,
+                          prob.mbr = 0.1,n.chains = 4,heating = 5,
+                          prior.choice = 1)
+
+#maximum scoring network using exact search (not MCMC based)
+dag <- mostprobable(score.cache = bsc.compute.marks)
+expect_equal(max(mcmc.out.marks$scores),fitabn(object = dag,data.df = marks,data.dists = dist.marks)$mlik, tol = 0.5)
+
+})
+
 #delete file temporary file
 if (file.exists("model.bug")) {
   file.remove("model.bug")}
